@@ -39,9 +39,9 @@ int main (int argc, char ** argv)
 
     strcpy (filename, "attributes.bp");
 
-    adios_init ("attributes.xml");
+    adios_init ("attributes.xml", comm);
 
-    adios_open (&adios_handle, "temperature", filename, "w", &comm);
+    adios_open (&adios_handle, "temperature", filename, "w", comm);
 #include "gwrite_temperature.ch"
     adios_close (adios_handle);
 
