@@ -40,12 +40,9 @@ int main (int argc, char ** argv)
 
         G = 2 * NX * size;
 
-	for (i = 0; i < NX; i++)
-	    t[i] = rank * NX + i;
-
 	strcpy (filename, "adios_globaltime.bp");
 
-	adios_init_noxml ();
+	adios_init_noxml (comm);
         adios_allocate_buffer (ADIOS_BUFFER_ALLOC_NOW, 10);
 
         int64_t       m_adios_group;
@@ -62,17 +59,24 @@ int main (int argc, char ** argv)
                      ,"", adios_integer
                      ,0, 0, 0);
 
-        adios_define_var (m_adios_group, "O"
-                     ,"", adios_integer
-                     ,0, 0, 0);
-
-        adios_define_var (m_adios_group, "temperature"
-                     ,"", adios_double
-                     ,"iter,NX", "G", "O");
+        /* have to define O and temperature as many times as we 
+           write them within one step (twice) */
+        for (it=0; it < 2; it++) {
+            adios_define_var (m_adios_group, "O"
+                         ,"", adios_integer
+                         ,0, 0, 0);
+    
+            adios_define_var (m_adios_group, "temperature"
+                         ,"", adios_double
+                         ,"iter,NX", "G", "O");
+        }
 
         for (it =0; it < 5; it++) {
 
-        	adios_open (&m_adios_file, "restart", filename, "a", &comm);
+            for (i = 0; i < NX; i++)
+                t[i] = rank + it*0.1 + 0.01;
+
+        	adios_open (&m_adios_file, "restart", filename, "a", comm);
         	adios_groupsize = 4 + 4 + 4 + NX * 8
                         	+ 4 + 4 + 4 + NX * 8;
         	adios_group_size (m_adios_file, adios_groupsize, &adios_totalsize);
@@ -84,10 +88,8 @@ int main (int argc, char ** argv)
         	adios_write(m_adios_file, "temperature", t);
 
         	for (i = 0; i < NX; i++)
-            		t[i]++;
+            		t[i] += 0.01;
 
-        	adios_write(m_adios_file, "NX", (void *) &NX);
-        	adios_write(m_adios_file, "G", (void *) &G);
         	O = rank * 2 * NX + NX;
         	adios_write(m_adios_file, "O", (void *) &O);
         	adios_write(m_adios_file, "temperature", t);
