@@ -20,7 +20,7 @@ int main (int argc, char ** argv)
 {
 	char        filename [256];
 	int         rank, size, i;
-	int         NX = 10; 
+	int         NX = 10;
 	double      t[NX];
 	MPI_Comm    comm = MPI_COMM_WORLD;
 
@@ -38,9 +38,9 @@ int main (int argc, char ** argv)
 
 	strcpy (filename, "adios_global.bp");
 
-	adios_init ("adios_global.xml");
+	adios_init ("adios_global.xml", comm);
 
-	adios_open (&adios_handle, "temperature", filename, "w", &comm);
+	adios_open (&adios_handle, "temperature", filename, "w", comm);
 	#include "gwrite_temperature.ch"
 	adios_close (adios_handle);
 
