@@ -30,6 +30,8 @@ int main (int argc, char ** argv)
     int         rank, size, i, j, npl, token;
     MPI_Comm    comm = MPI_COMM_WORLD;
     MPI_Status  status;
+    enum ADIOS_READ_METHOD method = ADIOS_READ_METHOD_BP;
+    ADIOS_SELECTION * sel;
     void * data = NULL;
     uint64_t start[1], count[1], bytes_read = 0;
 
@@ -37,21 +39,17 @@ int main (int argc, char ** argv)
     MPI_Comm_rank (comm, &rank);
     MPI_Comm_size (comm, &size);
 
-    ADIOS_FILE * f = adios_fopen ("adios_global_no_xml.bp", comm);
+    adios_read_init_method (method, comm, "verbose=3");
+
+    ADIOS_FILE * f = adios_read_open ("adios_global_no_xml.bp", method, 
+                                      comm, ADIOS_LOCKMODE_NONE, 0);
     if (f == NULL)
     {
         printf ("%s\n", adios_errmsg());
         return -1;
     }
 
-    ADIOS_GROUP * g = adios_gopen (f, "restart");
-    if (g == NULL)
-    {
-        printf ("%s\n", adios_errmsg());
-        return -1;
-    }
-
-    ADIOS_VARINFO * v = adios_inq_var (g, "temperature");
+    ADIOS_VARINFO * v = adios_inq_var (f, "temperature");
 
     /* Using less readers to read the global array back, i.e., non-uniform */
     uint64_t slice_size = v->dims[0]/size;
@@ -67,7 +65,10 @@ int main (int argc, char ** argv)
         return -1;
     }
 
-    bytes_read = adios_read_var (g, "temperature", start, count, data);
+    /* Read a subset of the temperature array */
+    sel = adios_selection_boundingbox (v->ndim, start, count);
+    adios_schedule_read (f, sel, "temperature", 0, 1, data);
+    adios_perform_reads (f, 1);
 
     if (rank > 0) {
         MPI_Recv (&token, 1, MPI_INT, rank-1, 0, comm, &status);
@@ -89,14 +90,11 @@ int main (int argc, char ** argv)
         MPI_Send (&token, 1, MPI_INT, rank+1, 0, comm);
     }
 
-    MPI_Barrier (comm);
-
     free (data);
 
-    adios_gclose (g);
-    adios_fclose (f);
-
-
+    adios_read_close (f);
+    MPI_Barrier (comm);
+    adios_read_finalize_method (method);
     MPI_Finalize ();
     return 0;
 }
